"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Worker = void 0;
const localforage = require("localforage");
class Worker {
    static withProviders(providers) {
        if (Array.isArray(providers)) {
            providers.forEach((provider) => {
                this.providers[(new provider).constructor.name] = provider;
            });
        }
        else {
            this.providers = providers;
        }
    }
    static getProvider(key) {
        if (this.providers[key]) {
            return this.providers[key];
        }
        throw new Error('Provider with key ' + key + ' not found!');
    }
    static addTask(task) {
        return __awaiter(this, void 0, void 0, function* () {
            let taskToStorage = task.serialize();
            let queueName = task.queue;
            let queue = yield this.getTasks(queueName);
            let newQueue;
            if (queue) {
                newQueue = queue;
            }
            else {
                newQueue = [];
            }
            newQueue.push(taskToStorage);
            yield this.saveTasks(task.queue, newQueue);
        });
    }
    static removeTasks(ids, queueName) {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            let queue = yield this.getTasks(queueName);
            ids.forEach((id) => {
                let taskIndex = queue.findIndex(task => {
                    task.id == id;
                    let taskProvider = this.getProvider(JSON.parse(task).storageKey);
                    let decoratedTask = taskProvider.decorate(JSON.parse(task));
                    if (decoratedTask.id === id) {
                        return task;
                    }
                });
                queue.splice(taskIndex, 1);
            });
            yield this.saveTasks(queueName, queue);
            resolve();
        }));
    }
    static up() {
        this.queues.forEach((queue) => {
            if (!this.intervalId) {
                this.intervalId = setInterval(() => {
                    this.runOnce(queue.key).then(() => {
                        return 'sucess';
                    }).catch((error) => {
                        return error;
                    });
                }, queue.timeCycle);
            }
            else {
                console.error('queue already running');
            }
        });
    }
    static runOnce(queueName = 'default') {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            console.info(queueName + ' - running worker cycle...');
            let queue = this.queues.find((queue => queue.key == queueName));
            if (queue) {
                let taskList = yield this.getTasks(queueName);
                //Se não houver um outro ciclo rodando
                if (!this.isRunning) {
                    if (taskList && taskList.length) {
                        this.runTask(queue, taskList[0], taskList, queueName).then((response) => {
                            resolve();
                        }).catch((error) => {
                            reject(error);
                        });
                    }
                }
                else {
                    reject('queueStillRunning');
                }
            }
            else {
                reject('queueNotFound');
            }
        }));
    }
    static runTask(queue, task, taskList, queueName) {
        return new Promise((resolve, reject) => {
            //Marca o ciclo como em execução
            queue.isRunning = true;
            this.isRunning = true;
            let taskProvider = this.getProvider(JSON.parse(task).storageKey);
            let decoratedTask = taskProvider.decorate(JSON.parse(task));
            if (decoratedTask) {
                decoratedTask.handle().then((response) => __awaiter(this, void 0, void 0, function* () {
                    let newTaskList = yield this.getTasks(queueName);
                    let taskIndex = newTaskList.findIndex(listedTask => {
                        let taskProvider = this.getProvider(JSON.parse(listedTask).storageKey);
                        let newDecoratedTask = taskProvider.decorate(JSON.parse(listedTask));
                        console.log(newDecoratedTask.id);
                        console.log(decoratedTask.id);
                        if (decoratedTask.id === newDecoratedTask.id) {
                            return task;
                        }
                    });
                    console.log(taskIndex + " sucesso");
                    if (taskIndex != -1) {
                        newTaskList.splice(taskIndex, 1);
                    }
                    yield this.saveTasks(decoratedTask.queue, newTaskList);
                    decoratedTask.response = response;
                    decoratedTask.afterHandle(decoratedTask);
                    this.afterRun(queue);
                    resolve();
                })).catch((error) => __awaiter(this, void 0, void 0, function* () {
                    decoratedTask.lastExecuted = new Date();
                    decoratedTask.tries++;
                    decoratedTask.errors = decoratedTask.errors ? decoratedTask.errors : [];
                    decoratedTask.errors.push(JSON.stringify(error, null, 2));
                    ;
                    decoratedTask.errors = decoratedTask.errors.slice(-5);
                    let newTaskList = yield this.getTasks(queueName);
                    let taskIndex = newTaskList.findIndex(listedTask => {
                        let taskProvider = this.getProvider(JSON.parse(listedTask).storageKey);
                        let newDecoratedTask = taskProvider.decorate(JSON.parse(listedTask));
                        if (decoratedTask.id === newDecoratedTask.id) {
                            return task;
                        }
                    });
                    console.log(taskIndex + " falha");
                    if (taskIndex != -1) {
                        newTaskList.splice(taskIndex, 1);
                    }
                    yield this.saveTasks(decoratedTask.queue, newTaskList);
                    yield this.addTask(decoratedTask);
                    this.afterRun(queue);
                    reject('taskError');
                }));
            }
            else {
                this.afterRun(queue);
                reject('runningQueue');
            }
        });
    }
    static afterRun(queue) {
        queue.isRunning = false;
        this.isRunning = false;
    }
    static saveTasks(queueName, queueData) {
        return __awaiter(this, void 0, void 0, function* () {
            let dataToStorage = JSON.stringify(queueData);
            yield localforage.setItem('queue.' + queueName, dataToStorage);
        });
    }
    static getTasks(queueName) {
        return __awaiter(this, void 0, void 0, function* () {
            let queue = yield localforage.getItem('queue.' + queueName);
            if (queue) {
                let queueParsed = JSON.parse(queue);
                return queueParsed;
            }
            else {
                return [];
            }
        });
    }
    /**
     * Derruba o worker
     */
    static stop() {
        return new Promise((resolve, reject) => {
            this.queues.map((queue) => {
                let recursiveStop = () => {
                    setTimeout(() => {
                        //Se estiver rodando um ciclo, espera até acabar
                        if (this.isRunning) {
                            recursiveStop();
                        }
                        else {
                            //Se não estiver executando, para o worker
                            clearInterval(this.intervalId);
                            this.intervalId = undefined;
                            resolve();
                        }
                    }, 500);
                };
                recursiveStop();
            });
        });
    }
}
exports.Worker = Worker;
Worker.queues = [{ key: 'default', timeCycle: 5000, isRunning: false }];
Worker.providers = {};
Worker.isRunning = false;
//# sourceMappingURL=Worker.js.map